#!/usr/bin/perl -T
#
# NOTE: This script runs in Perl's "Taint" mode ('-T' flag in first line).
#


use strict;
use Cwd  qw(abs_path);
use File::Basename;

my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};

my $q = new CGI;


# Untaint environment variables
#
$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";
$ENV{'CDPATH'} = '.';
$ENV{'ENV'} = '';


# URL to this script
my $SELFURL = $q->url();


print $q->header('text/html');
print '
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
            "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>'.$CONFIG{'APPNAME'}.': Help</title>
<link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
<base target="_parent">
</head>
<body>
  '.$CONFIG{'HTML_HEADER'}.'
  
  <h2>'.html_pretty('Help').'</h2>
  <div class="sidemargin">
  
  <p>&nbsp;</p>
  
  <h3>'.html_pretty('Before you begin').'</h3>
  <div class="sidemargin">
  
  <p>&nbsp;</p>

    <h4>'.html_pretty('System Requirements').'</h4>
    <div class="sidemargin">
    
    <ul>
      <li>A browser that supports frames, JavaScript and Java. The author recommends <a target="_blank" href="http://www.mozilla.com/firefox/">Mozilla Firefox</a>.</li>
      <li><a target="_blank" href="http://www.java.com/getjava/">Sun Java</a> or compatible.</li>
    </ul>
  
    </div> <!-- End sidemargin (level h4) -->
    <h4>'.html_pretty('Input Data Format').'</h4>
    <div class="sidemargin">
    ';
    
if ($CONFIG{'APPNAME'} eq 'Completionist')
{
    print '
    <p>Your input should be a membrane protein sequence and a structure. Make sure your input is in the correct format.</p>
    <ul>
      <li>Input sequences must be provided as pure amino acid sequences, without titles or comments (e.g. &quot;ACDEFG&quot;).</li>
      <li>Input structures must be in <a target="_blank" href="http://en.wikipedia.org/wiki/Protein_Data_Bank_(file_format)">PDB</a> format.</li>
    </ul>
    ';
}
else
{
    print '
    <p>Your input should be a membrane protein sequence or structure. Make sure your input is in the correct format.</p>
    <ul>
      <li>Input sequences must be in <a target="_blank" href="http://en.wikipedia.org/wiki/Fasta_format">FASTA</a> or PIR format.</li>
      <li>Input structures must be in <a target="_blank" href="http://en.wikipedia.org/wiki/Protein_Data_Bank_(file_format)">PDB</a> format.</li>
    </ul>
    ';
}

print '
    </div> <!-- End sidemargin (level h4) -->
  </div> <!-- End sidemargin (level h3) -->
  <h3>'.html_pretty('Step by step').'</h3>
  <div class="sidemargin">
  
  <p>&nbsp;</p>

    <h4>'.html_pretty('Submitting your query').'</h4>
    <div class="sidemargin">
    
    <ol>';

if ($CONFIG{'APPNAME'} eq 'MEDELLER')
{
    print '
      <li>
        Go to the box entitled &quot;Input Target-Template Alignment&quot;. Pick a way to provide your input.
        You can either upload a file or copy and paste your data into a text area. Either way, make sure your
        input sequences are in FASTA or PIR format. Your target and template sequences should be labelled as
        "&gt;target" and "&gt;template", respectively (without the quotes).
      </li>
      <li>
        Go to the box entitled &quot;Input Template Structure&quot;. Pick a way to provide your input. 
        You can either enter a 5-character PDB code (the last character is the chain identifier), upload a file
        or copy and paste your data into a text area. Make sure that any data you provide here is in PDB format.
      </li>
      ';
}
elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
{
    print '
      <li>
        Look at the first box entitled &quot;Input type&quot;. If you only have the sequence of your protein, choose &quot;sequence&quot;.
        If you have the structure of your protein, choose &quot;structure&quot;.
      </li>
      <li>
        Move on to the second box entitled &quot;Input&quot;. Pick a way to provide your input.
        You can either provide a PDB id, upload a file or copy and paste your data into a text area.
        The first time you try iMembrane, just leave the first option selected and enter your PDB code into the text field.
      </li>
    ';
}
elsif ($CONFIG{'APPNAME'} eq 'Completionist')
{
    print '
      <li>
        Go to the box entitled &quot;Complete target sequence&quot;. Copy and paste your data into a text area.
        Make sure your input contains only the amino acid letters (e.g. &quot;ACDEFG&quot;) and no titles or comments.
      </li>
      <li>
        Go to the box entitled &quot;Incomplete model structure&quot;. Pick a way to provide your input. 
        You can either upload a file or copy and paste your data into a text area. Make sure that any data you provide
        here is in PDB format.
      </li>
      ';
}

if ($CONFIG{'APPNAME'} eq 'MEDELLER' or $CONFIG{'APPNAME'} eq 'iMembrane')
{
    print '
      <li>
        Go to the box entitled &quot;Options&quot;. If you are submitting a query for the first time, do not do anything here,
        simply leave the default settings unchanged and move on to the next step.<br>';
        
    if ($CONFIG{'APPNAME'} eq 'MEDELLER')
    {
      print '
        If you are running MEDELER, you have the option of choosing the type of model you will generate.
        A more complete model takes longer to generate than just the core model. In case you are submitting a protein that is not an
        integral membrane protein, you may wish to activate the option to &quot;Start modelling from a bigger minimal core&quot;.<br>';
    }
    
    print '
        You have a choice of different iMembrane search methods. Your choice influences the speed of the search and the number of hits found.
        The different choices are briefly explained below the radio buttons in the &quot;Options&quot; box. In general, it is fine to leave the search
        in &quot;auto&quot; mode.
        You may also choose different cut-off values for your search. The default values are quite loose, but still manage to filter out absolutely
        terrible hits. In any case, your search results are ranked. If you are running MEDELLER, only the top hit will be used to annotate your
        template protein, which is then in turn used to annotate the models of your target protein.
      </li>';
}
elsif ($CONFIG{'APPNAME'} eq 'Completionist')
{
    print '
      <li>
        Go to the box entitled &quot;Options&quot;. Choose the type of protein you are modelling. This can be either a soluble or membrane protein, or an immunoglobulin-like protein (e.g. an antibody). This choice will determine which fragment database will be used for the FREAD loop modelling step. Any loops not modellable with FREAD will be filled using the MODELLER ab initio method.
      </li>';

}

print '
      <li>
        The last box: &quot;Submit&quot;. Press the &quot;Submit&quot; button once, and only once please, to submit your query.
        Your browser may stay inert from a few seconds up to several minutes (in some cases), while it uploads your input to our web site.
        If you are unsure whether you have successfully clicked the button, have a look at your browser&#39;s status bar. If this starts
        with something like &quot;Sending&quot;, &quot;Waiting&quot; or &quot;Uploading&quot; then everything is fine and you simply need to wait.
        The screen should automatically refresh and forward you to your results.
      </li>
    </ol>
    
    </div> <!-- End sidemargin (level h4) -->
    <h4 id="viewresults">'.html_pretty('Viewing your results').'</h4>
    <div class="sidemargin">
    ';

if ($CONFIG{'APPNAME'} eq 'MEDELLER' or $CONFIG{'APPNAME'} eq 'iMembrane')
{
  print '
    <p>The results screen consists of 4 main frames:</p>
    
    <ul>
      <li>Top-left: The <strong>Query</strong> and <strong>Match</strong> structure viewers.</li>
      <li>Top-right: The <strong>Sequence Alignment</strong> viewer.</li>
      <li>Bottom half: In MEDELLER mode, this area lists all your generated models and some links where you can view or download these. In iMembrane mode, the <strong>Results</strong> list shows all the hits for your query in a tabular format, one query/hit pair per row.
      The links in the hits table control what is being shown in the above 3 frames and allow you to download individual hits to your computer.</li>
    </ul>
    
      <h5>The structure viewers</h5>
      <div class="sidemargin">
        
        <p>These frames show three-dimensional representations of your models (MEDELLER) or your annotated input structure and its database match (iMembrane). Colours represent the annotation produced by iMembrane.
        In MEDELLER mode, the left structure is coloured by modelling confidence, i.e. the model core has high confidence (coloured blue) whereas
        loop coordinates have lower confidence and ab-initio loops have the lowest conficence (coloured red).<br>
        You can turn the molecule by clicking and dragging it with the left mouse button. Right-clicking gives various view options.
        Consult the <a target="_blank" href="http://jmol.sourceforge.net/docs/">Jmol manual</a> for details.</p>
      
      </div> <!-- End sidemargin (level h5) -->
      <h5>The sequence alignment viewer</h5>
      <div class="sidemargin">
      
        <p>This shows the sequence of your input protein, aligned to its corresponding database hit (iMembrane) or your model aligned to your template (MEDELLER).
        The lines entitled &quot;membrane layer&quot; and &quot;membrane contact&quot; show the annotation for the database hit, as produced by iMembrane.<br>
        The colour key below the sequences explains the meaning of the colouring in both the sequence alignment as well as the structure viewers.
        Refer to the section &quot;Understanding iMembrane annotation&quot; below for details.</p>
      
      </div> <!-- End sidemargin (level h5) -->
      <h5 id="resultlist">The results list</h5>
      <div class="sidemargin">
      
        <p>The text at the top of the Results list mentions your result&#39;s Result ID. Make a note of this or bookmark the link to your result page to come back to it later. Note, however, that your result will only be saved for 24 hours before being deleted from the server.</p>
        <p>Clicking on the links named &quot;Layer&quot; or &quot;Contact&quot; in each row will change the structures and sequences being displayed in the structure viewer and sequence viewer frames. For the exact meaning of &quot;Layer&quot; and &quot;Contact&quot;, refer to the section &quot;Understanding iMembrane annotation&quot; below.</p>
        
        <p>In MEDELLER mode, the list of generated models is shows here: <strong>core</strong> (a reliable subset of the template co-ordinates), <strong>hiacc</strong> (high accuracy model, with high accuracy database loops), <strong>hicov</strong> (high coverage model, with high and low accuracy database loops) and <strong>complete</strong> (the complete model with high and low accuracy database loops and all remaining gaps filled with ab initio methods). The complete model is the only model containing sidechains, terminal residues not present in the input model as well as very long gap regions (>30 residues). The largest generated model (usually the complete model) is shown by default.</p>
        
        <p>The result table (iMembrane only) has several columns. Their meaning is explained below.</p>
        <ul>
          <li id="column_dbid"><strong>dbid:</strong> The name given to the particular database molecule that matched the above query molecule. This is simply a PDB code, simulation number, chain identifier and a duplicate chain number (some chains appear more than once in a PDB structure, e.g. via rotational operators). The simulation number identifies the molecular dynamics simulation that underlies iMembrane&#39;s annotation for this molecule. Clicking on the dbid will bring up the particular CGDB entry associated with this molecule.</li>
          <li id="column_e"><strong>E-value:</strong> The BLAST E-value corresponding to this match. Values below 1.0E-20 (ten to the power of -20) usually assure a good quality structure-based annotation, while values below 1.0E-40 (ten to the power of -40) tend to assure a good sequence-based annotation. Higher values do not guarantee a bad annotation.</li>
          <li id="column_tm"><strong>TM-score:</strong> The TM-score corresponding to the final alignment of the query and match molecules. Values above 0.5 tend to indicate an acceptable match.</li>
          <li id="column_pannot"><strong>pannot:</strong> The percentage of your query protein that could be annotated using this match (measured on the sequence-based &quot;membrane contact&quot; annotation). Currently this is always equal to the coverage. See below.</li>
          <li id="column_pcov"><strong>pcov:</strong> The percentage coverage, i.e. fraction of query residues aligned 
          [100 * count(aligned residues) / length(query)].<br>
          pid &lt;= pcov.</li>
          <li id="column_pid"><strong>pid:</strong> The percentage sequence identity between the query and match molecules, computed from the final alignment between the two. This number is relative to the length of the query molecule 
          [100 * count(identical residues) / length(query)].<br>
          pid &lt;= pcov.</li>
          <li id="column_rmsd"><strong>RMSD:</strong> The Root Mean Square Deviation of the structural alignment between the query and match proteins. This is not an indicator of annotation quality.</li>
          <li id="column_view"><strong>View Results:</strong> This column contains links to change the molecules shown in the &quot;Query&quot;, &quot;Match&quot; and &quot;Sequence Alignment&quot; frames. Clicking on the &quot;Contact&quot; or &quot;Layer&quot; link in any of the rows will display the corresponding query/match pair, coloured by either &quot;membrane layer&quot; or &quot;membrane contact&quot;. See the section &quot;Understanding iMembrane annotation&quot; below for detail on the meaning of these annotation types.</li>
          <li id="column_download"><strong>Download Results:</strong> This column contains links to download individual query/match pairs and all associated files to your computer. Clicking on the link will download a g-zipped tar archive (file extension &quot;tar.gz&quot;). Make sure to give the archive an appropriate name if you intend to keep it. Most modern archiving programmes support tar.gz archives. The author recommends <a target="_blank" href="http://www.7-zip.org/">7-zip</a> to Windows users. Linux and Mac users can unzip the archive with the &quot;tar zxf &lt;filename&gt;&quot; command.</li>
        </ul>
    
      </div> <!-- End sidemargin (level h5) -->
    </div> <!-- End sidemargin (level h4) -->
    <h4>'.html_pretty('Understanding iMembrane annotation').'</h4>
    <div class="sidemargin">
  
      <p>The annotation produced by iMembrane comes in two variants:</p>
      <ul>
        <li>Membrane contact</li>
        <li>Membrane layer</li>
      </ul>
      
      <h5>Membrane contact</h5>
      <div class="sidemargin">
      
        <p>This is the most basic type of annotation. The data is merely a descretised version of the annotation obtained from the original molecular dynamics experiments performed on the database match protein. The annotation distinguishes between residues that are in direct contact with the inner (lipid tails) or outer (polar head groups) membrane layers and those residues that are not at all in contact with the membrane. By aligning the query sequence to the match sequence, this annotation can be transferred over to the query protein, in regions where the two proteins are similar.</p>
      
      </div> <!-- End sidemargin (level h5) -->
      <h5>Membrane layer</h5>
      <div class="sidemargin">
      
        <p>This is a more abstract and simplified model of the membrane. By using the above membrane contact data, one can fit parallel plains onto the database protein. These planes represent the limits between the different layers of the lipid bilayer. The protein&#39;s residues are essentially classified into three categories: Those in the middle layer of the membrane, where the hydrophobic tails of the membrane lipids reside; the residues in the outer membrane layers, where the polar head groups of the membrane lipids reside; the residues outside the membrane. <span class="note">Note that, unlike the contact annotation, being in a certain layer does not mean that the residue in question is actually in contact with this environment. For example, residues in the core of the protein can be classified as being in the middle of the membrane, if the are positioned on the same level as the lipid tails along the bilayer normal.</span></p>
      
      </div> <!-- End sidemargin (level h5) -->';
}
elsif ($CONFIG{'APPNAME'} eq 'Completionist')
{
    print '
    <p>The results screen consists of 2 main frames:</p>
    
    <ul>
      <li>Left: The model structure viewer.</li>
      <li>Right: This area lists all your generated models and some links where you can view or download these.</li>
    </ul>
    
      <h5>The structure viewer</h5>
      <div class="sidemargin">
        
        <p>This frame shows three-dimensional representations of your models. The structure is coloured by modelling confidence, i.e. the model core has high confidence (coloured blue) whereas
        loop coordinates have lower confidence and ab-initio loops have the lowest conficence (coloured red).<br>
        You can turn the molecule by clicking and dragging it with the left mouse button. Right-clicking gives various view options.
        Consult the <a target="_blank" href="http://jmol.sourceforge.net/docs/">Jmol manual</a> for details.</p>
      
      </div> <!-- End sidemargin (level h5) -->
      
      <h5 id="resultlist">The results list</h5>
      <div class="sidemargin">
      
        <p>The text at the top of the Results list mentions your result&#39;s Result ID. Make a note of this or bookmark the link to your result page to come back to it later. Note, however, that your result will only be saved for about 24 hours before being deleted from the server.</p>
        
        <p>The list of models shows several models of increasing coverage: <strong>core</strong> (your input), <strong>hiacc</strong> (high accuracy model, with high accuracy database loops), <strong>hicov</strong> (high coverage model, with high and low accuracy database loops) and <strong>complete</strong> (the complete model with high and low accuracy database loops and all remaining gaps filled with ab initio methods). The complete model is the only model containing sidechains, terminal residues not present in the input model as well as very long gap regions (>30 residues). These are all co-ordinates that are very hard to model and are usually inaccurate. We do not attempt to model them and leave it to MODELLER to provide placeholder co-ordinates, for your convenience only. The complete model is shown by default.</p>

        <p>If you are interested in high-accuracy sidechains, we suggest you try specialised sidechain modelling sofware, such as <a href="http://dunbrack.fccc.edu/scwrl4/" target="_blank">SCWRL</a>.</p>
        
      </div> <!-- End sidemargin (level h5) -->
  ';
}


print '
    </div> <!-- End sidemargin (level h4) -->
  </div> <!-- End sidemargin (level h3) -->
  
  </div> <!-- End sidemargin (level h2) -->
  <p>&nbsp;</p>
  
  '.$CONFIG{'HTML_FOOTER'}.'
</body>
</html>
';
