#!/usr/bin/perl -TW
#
# This CGI programme takes protein PDB codes/files or FASTA protein sequences as input.
#
# It saves the input as an iMembrane JOB, generates a JOB ID and forwards the user to
# the waiting page.
#
# NOTE: This script runs in Perl's "Taint" mode ('-T' flag in first line).
#


use strict;
use Cwd  qw(abs_path);
use File::Basename;
#~ use Digest::MD5  qw(md5_hex);
use CGI;
use File::Glob ':glob';


my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};

my $q = new CGI;


###################################
#### SETTINGS AND PREPARATIONS ####
###################################

#my $LOGFILE = 'submission.log.txt';



#############################
#### CREATE AN EMPTY JOB ####
#############################

my $result_id = time().'_'.sprintf('%06d', rand(999999));
my $JOBPATH   = $CONFIG{'JOBSDIR'}."/$result_id";

# Create output directory
system("mkdir -m0775 -p $JOBPATH");
unless (-d $JOBPATH)
{
  server_error("Could not create output directory: $JOBPATH");
}
system("chmod 0775 $JOBPATH");
  
chdir($JOBPATH);
umask(0002); # chmod all new files to 0775


#server_error("$ENV{'PATH'}");


################################
#### FILL THE JOB WITH DATA ####
################################


## sequence input
#
my $seq_text       = get_input('seq_text', qr/^[A-Za-z\s]+$/);
$seq_text =~ s/\s+//g;

## structure input
#
my $struc_method   = get_input('struc_method', qr/^file$|^text$/);
my $struc_text     = $q->param('struc_text');
my @struc_files    = $q->param('struc_file');

my $fread_db       = get_input('fread_db', qr/^mem$|^sol$|^ab$/);

my $template_file  = "$JOBPATH/model.pdb";


## Save structure input file
## and write command line option strings to file
#

create_input_file($template_file, $struc_method, undef, $struc_text, \@struc_files);
if (!(-f $template_file))
{
  finish("Insufficient input received!");
}

write_file("$JOBPATH/$CONFIG{'OPTIONSFILE'}", "$CONFIG{'APPCOMMAND'} --atm --refine_sidechains --dbtype '$fread_db' --seq '$seq_text' --struc '$template_file'");
#write_file("$JOBPATH/$CONFIG{'OPTIONSFILE'}", "--atm --dbtype '$fread_db' --seq '$seq_text' --struc '$template_file'");


## Adjust permissions of saved files so that they are readable by everyone
## (the daemon process that executes jobs usually runs as a different user)
#

system("chmod 0664 $JOBPATH/*");


finish('OK', 'queued');



#########################################################################



sub create_input_file
{
    my ($output_filename, $input_method, $input_pdbid, $input_text, $input_files) = @_;
    
    if ($input_method eq 'text')
    {
      unless ($input_text)
      {
        finish("ERROR: No input! (Input method = '$input_method')\n");
      }
      
      open(my $textfile, ">$output_filename");
      print $textfile $input_text;
      close($textfile);

      unless (-f "$output_filename")
      {
        finish("ERROR: Failed writing input to file! (Input method = '$input_method', Output file = '$output_filename')\n");
      }
    }
    elsif ($input_method eq 'pdbid')
    {
      unless ($input_pdbid)
      {
        finish("ERROR: No input! (Input method = '$input_method')\n");
      }
      $input_pdbid = lc(substr($input_pdbid, 0, 4)) . substr($input_pdbid, 4);
      
      # A bit silly to call getpdbchain with a path, but $ENV{'PATH'} seems to get ignored here
      system("$CONFIG{'BINDIR'}/getpdbchain $input_pdbid 1>getpdbchain.log.txt 2>&1");
      open(my $getpdblog, 'getpdbchain.log.txt') or finish("ERROR: Could not open getpdbchain.log.txt");
      my $log = '';
      {
        local $/ = undef;
        $log = <$getpdblog>;
      }
      close($getpdblog);
      
      if (!(-f "$input_pdbid.pdb"))
      {
        finish("Unable to retrieve PDB structure: $input_pdbid\nMessage:\n\n$log\n");
      }
      
      system("mv -f ${input_pdbid}.pdb $output_filename") and finish("ERROR: Unable to rename PDB structure: $input_pdbid");
      if (!(-f "$output_filename"))
      {
        finish("ERROR: Unable to rename PDB structure: $input_pdbid\n");
      }
    }
    elsif ($input_method eq 'file')
    {
      my ($filename) = @$input_files;
      if (!$filename && $q->cgi_error)
      {
        finish($q->header(-status=>$q->cgi_error));
      }
      if (!defined($filename))
      {
        finish("Could not read uploaded file\n");
      }
      if (!$filename)
      {
        finish("No input. Make sure to enter a sequence or select a file to upload.\n");
      }
      
    #       my $type = $q->uploadInfo($filename)->{'Content-Type'};
    #       if ($type !~ m%^text/%)
    #       {
    #         finish("Input files must be text files!");
    #       }
      
      if ($filename !~ m%([^/]+)$%)
      {
        finish("Illegal filename format: $filename\n");
      }
      
      seek($filename,0,0); # reset postion to beginning of file.
      open (my $newloc, ">$output_filename") || finish("Unable to copy input file to output file: $output_filename\n"); # open new file for writing
      my $buffer;
      while (my $bytesread=read($filename,$buffer,1024))
      {
        print $newloc $buffer; # copy contents to new file
      }
      close($newloc);
    }
    else
    {
      finish("Illegal input method : '$input_method'")
    }
}


sub set_status
{
    my ($jobpath, $statusname, $message) = @_;
    unless(defined($message))
    {
      $message = 'OK';
    }
    write_file("$jobpath/status.$statusname", $message);
}


sub finish
{
    my ($message, $status) = @_;
    unless(defined($status))
    {
      $status = 'refused';
    }
    set_status($JOBPATH, $status, $message);
    
    print $q->redirect("$CONFIG{'QUEUEURL'}?app=$CONFIG{'APPNAME'}&result_id=$result_id");
    goto END;
}

END:
1;

