#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import sys
import os
import subprocess
import tempfile
import shutil
import re
from glob import iglob

from prosci.util.command import Command
from prosci.xtal.superpose import superpose, SuperpositionFailed


def _resolve(fname):
  for x in iglob(fname):
    return x
  raise IOError("No such file or directory: '%s'"%fname)


"""
TYPICAL END OF phenix.find_alt_orig_sym_mate OUTPUT:

log written to AltOrigSymMLAD_3TLP_stage0012_edited.log
MLAD of -0.029259 for chain A in dir2/stage0012_edited.pdb with chain A in dir1/3TLP.pdb is achieved with solution 0
MLAD of -0.047777 for chain B in dir2/stage0012_edited.pdb with chain B in dir1/3TLP.pdb is achieved with solution 0
"""


re_chainmatch = re.compile("MLAD of ([+-]?[\d\.]+) for chain (.?) in .* with chain (.?) in .* is achieved with solution \d+")

def symmate(freference, fmoving, foutput, timeout=sys.maxint):
  command = "phenix.find_alt_orig_sym_mate moving_pdb='%s' fixed_pdb='%s'"%(fmoving, freference)

  print command
  
  tempdir = tempfile.mkdtemp(prefix="symmate")
  try:
    c = Command(command)
    c.run(timeout, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, cwd=tempdir)
    
    if c.timedout:
      raise SuperpositionFailed("phenix.find_alt_orig_sym_mate job timed out")
    elif c.process.returncode != 0:
      raise SuperpositionFailed("phenix.find_alt_orig_sym_mate crashed... probably")
    
    fmoved = _resolve(os.path.join(tempdir, "MinMLAD_*.pdb"))
    shutil.move(fmoved, foutput)
  finally:
    shutil.rmtree(tempdir)
  
  matchinfo = None
  for line in c.out.splitlines():
    if matchinfo is not None:
      mlad, chain_moving, chain_reference = re_chainmatch.match(line).groups()
      matchinfo.append([float(mlad), chain_moving.strip(), chain_reference.strip()])
    if line.startswith("log written to"):
      matchinfo = []
  
  return matchinfo, c.err


def superpose_symmates(freference, fmoving, foutput, timeout=sys.maxint):
  tmp_fh_output, tmp_fname_output = tempfile.mkstemp(suffix=".pdb", prefix="symmate_output")
  chainmatch, err_symmate = symmate(freference, fmoving, tmp_fname_output, timeout=timeout)
  
  try:
    tmp_fh, tmp_fname = tempfile.mkstemp(suffix=".pdb", prefix="superpose_output")
    
    """
    try:
      resmatch, err2 = superpose(freference, fmoving, tmp_fname, timeout=timeout)
    finally:
      os.remove(tmp_fname)
    
    chaindict = {}
    for c_mov, c_ref, score in chainmatch:
      chaindict[c_mov] = c_ref
    
    allok = True
    for resright, resleft, dist in resmatch:
      if resright and resleft:
        chain1 = resright[0]
        chain2 = resleft[0]
        if chaindict[chain1] != chain2:
          allok = False
          break
    
    if not allok:
    """
    
    warning = "WARNING! Not all chains are matched on the same origin." in err_symmate
    
    chainmatch.sort()
    
    resmatch = []
    err_superpose = ""
    chains_used_mov = []
    chains_used_ref = []
    skipped_chains = False
    for score, c_mov, c_ref in chainmatch:
      if c_mov in chains_used_mov or c_ref in chains_used_ref:
        skipped_chains = True
        continue
      chains_used_mov.append(c_mov)
      chains_used_ref.append(c_ref)
      
      try:
        chain_resmatch, chain_err = superpose(freference, fmoving, tmp_fname, sel_reference=c_ref, sel_moving=c_mov, timeout=timeout)
      finally:
        if os.path.exists(tmp_fname):
          os.remove(tmp_fname)
      resmatch.extend(chain_resmatch)
      err_superpose += chain_err
    
    if not skipped_chains:
      shutil.move(tmp_fname_output, foutput)
    else:
      fin = open(tmp_fname_output)
      fout = open(foutput, "w")
      for line in fin:
        if not (line[:6] in ("ATOM  ", "HETATM", "ANISOU")):
          fout.write(line)
        elif line[21:22].strip() in chains_used_mov:
          fout.write(line)
      fout.close()
      fin.close()
    
  finally:
    if os.path.exists(tmp_fname_output):
      os.remove(tmp_fname_output)
  
  return resmatch, err_symmate + err_superpose
